==> haystack-0.4.1.tar.gz: lib/src/HCoord.dart <==
///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hcoord;

import 'HVal.dart';
import 'ParseError.dart';
import 'str-utils.dart';
import 'package:quiver/core.dart';
import 'package:intl/intl.dart';

/// HBool defines singletons for true/false tag values.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HCoord extends HVal {

  /** Construct from basic fields */
  factory HCoord(num lat, num lng) {
    int ulat = (lat * 1000000.0).toInt();
    int ulng = (lng * 1000000.0).toInt();

    if (ulat < -90000000 || ulat > 90000000) throw new ArgumentError("Invalid lat > +/- 90");
    if (ulng < -180000000 || ulng > 180000000) throw new ArgumentError("Invalid lng > +/- 180");
    return new HCoord._(ulat, ulng);
  }

    /** Parse from string fomat "C(lat,lng)" or raise ParseException */
  factory HCoord.fromString(String s) {
    try {
      if (!s.startsWith("C(")) throw new Error();
      if (!s.endsWith(")")) throw new Error();
      int comma = s.indexOf(',');
      if (comma < 3) throw new Error();
      String lat = s.substring(2, comma);
      String lng = s.substring(comma+1, s.length-1);

      return new HCoord(double.parse(lat), double.parse(lng));
    } catch (e) {
      throw new ParseError(s);
    }
  }

  /** private constructor */
  HCoord._(this.ulat, this.ulng);

  /** Return if given latitude is legal value between -90.0 and +90.0 */
  static bool isLat(double lat) => -90.0 <= lat && lat <= 90.0;

  /** Return if given is longtitude is legal value between -180.0 and +180.0 */
  static bool isLng(double lng) => -180.0 <= lng && lng <= 180.0;

  /** Latitude in decimal degrees */
  double lat() => ulat / 1000000.0;

  /** Longtitude in decimal degrees */
  double lng() => ulng / 1000000.0;

  /** Latitude in micro-degrees */
  final int ulat;

  /** Longitude in micro-degrees */
  final int ulng;

  /** Hash is based on lat/lng */
  int _hashCode;
  int get hashCode {
    if (_hashCode == null) {
      _hashCode = hash2(ulat, ulng);
    }
    return _hashCode;
  }

  /** Equality is based on lat/lng */
  bool operator ==(Object that) => that is HCoord && ulat == (that as HCoord).ulat && ulng == (that as HCoord).ulng;

  /** Represented as "C(lat,lng)" */
  String _toZinc;
  String toZinc() {
    if (_toZinc == null) {
      StringBuffer s = new StringBuffer();
      s.write("C(");
      _uToStr(s, ulat);
      s.write(',');
      _uToStr(s, ulng);
      s.write(")");
      _toZinc = s.toString();
    }
    return _toZinc;
  }

  void _uToStr(StringBuffer s, int ud) {
    if (ud < 0) {
      s.writeCharCode(MINUS_CHAR);
      ud = -ud;
    }

    if (ud < 1000000.0) {
      s.write(new NumberFormat("0.0#####", 'en_US').format(ud/1000000.0));
    }
    else
    {
      String x = ud.toString();
      int dot = x.length - 6;
      int end = x.length;
      while (end > dot+1 && x.codeUnitAt(end-1) == ZERO_CHAR) --end;
      for (int i=0; i<dot; ++i) s.writeCharCode(x.codeUnitAt(i));
      s.writeCharCode(PERIOD_CHAR);
      for (int i=dot; i<end; ++i) s.writeCharCode(x.codeUnitAt(i));
    }
  }
}

==> haystack-0.4.19.tar.gz: lib/src/HCoord.dart <==
///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.hcoord;

import 'HVal.dart';
import 'str-utils.dart';
import 'package:quiver/core.dart';
import 'package:intl/intl.dart';
import 'io/HZincReader.dart';

/// HBool defines singletons for true/false tag values.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HCoord extends HVal {

  /** Construct from basic fields */
  factory HCoord.from( Object o, [num lng]) {
    if ( o is String ) return new HCoord.fromString(o);
    if ( o is num ) return new HCoord(o, lng);
    return null;
  }

  factory HCoord(num lat, num lng) {
    int ulat = (lat * 1000000.0).toInt();
    int ulng = (lng * 1000000.0).toInt();

    return new HCoord._(ulat, ulng);
  }

    /** Parse from string fomat "C(lat,lng)" or raise ParseException */
  factory HCoord.fromString(String s) {
    return new HZincReader(s).readVal();
  }

  /** private constructor */
  HCoord._(this.ulat, this.ulng) {
    if (ulat < -90000000 || ulat > 90000000) throw new ArgumentError("Invalid lat > +/- 90");
    if (ulng < -180000000 || ulng > 180000000) throw new ArgumentError("Invalid lng > +/- 180");
  }

  /** Return if given latitude is legal value between -90.0 and +90.0 */
  static bool isLat(double lat) => -90.0 <= lat && lat <= 90.0;

  /** Return if given is longtitude is legal value between -180.0 and +180.0 */
  static bool isLng(double lng) => -180.0 <= lng && lng <= 180.0;

  /** Latitude in decimal degrees */
  double lat() => ulat / 1000000.0;

  /** Longtitude in decimal degrees */
  double lng() => ulng / 1000000.0;

  /** Latitude in micro-degrees */
  final int ulat;

  /** Longitude in micro-degrees */
  final int ulng;

  /** Hash is based on lat/lng */
  int _hashCode;
  int get hashCode {
    if (_hashCode == null) {
      _hashCode = hash2(ulat, ulng);
    }
    return _hashCode;
  }

  /** Equality is based on lat/lng */
  bool operator ==(Object that) => that is HCoord && ulat == that.ulat && ulng == that.ulng;

  /** Return "c:lat,lng" */
  String toJson() => (new StringBuffer()
      ..write("c:")
      ..write(_uToStr(ulat))
      ..write(',')
      ..write(_uToStr(ulng))).toString();

  /** Represented as "C(lat,lng)" */
  String toZinc() {
    StringBuffer s = new StringBuffer()
      ..writeAll( ["C(", _uToStr(ulat), ',', _uToStr(ulng), ")" ], "" );
    return s.toString();
  }

  static String _uToStr(int ud) {
    StringBuffer s = new StringBuffer();

    if (ud < 0) { s.writeCharCode(MINUS_CHAR); ud = -ud; }

    if (ud < 1000000.0) {
      s.write(new NumberFormat("0.0#####", 'en_US').format(ud/1000000.0));
    } else {
      String x = ud.toString();
      int dot = x.length - 6;
      int end = x.length;
      while (end > dot+1 && x.codeUnitAt(end-1) == ZERO_CHAR) --end;
      for (int i=0; i<dot; ++i) s.writeCharCode(x.codeUnitAt(i));
      s.writeCharCode(PERIOD_CHAR);
      for (int i=dot; i<end; ++i) s.writeCharCode(x.codeUnitAt(i));
    }

    return s.toString();
  }
}
